/*
 * Copyright (c) 2006-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: IfNode.java,v 1.10 2007/05/21 20:08:07 sshirasu Exp $
 */

package edu.indiana.extreme.xbaya.graph.system;

import java.util.List;

import org.xmlpull.infoset.XmlElement;

import edu.indiana.extreme.xbaya.component.system.IfComponent;
import edu.indiana.extreme.xbaya.component.system.SystemComponentDataPort;
import edu.indiana.extreme.xbaya.graph.DataPort;
import edu.indiana.extreme.xbaya.graph.Graph;
import edu.indiana.extreme.xbaya.graph.GraphException;
import edu.indiana.extreme.xbaya.graph.GraphSchema;
import edu.indiana.extreme.xbaya.graph.gui.NodeGUI;
import edu.indiana.extreme.xbaya.graph.system.gui.IfNodeGUI;

/**
 * @author Satoshi Shirasuna
 */
public class IfNode extends SystemNode {

    private static final String XPATH_TAG_NAME = "xpath";

    // private static final MLogger logger = MLogger.getLogger();

    private IfNodeGUI gui;

    private String xpath;

    /**
     * Creates a InputNode.
     * 
     * @param graph
     */
    public IfNode(Graph graph) {
        super(graph);

        // Set the default to $0, which means that the input is boolean and
        // xpath uses it as it is.
        this.xpath = "$0";
    }

    /**
     * Constructs an IfNode.
     * 
     * @param nodeElement
     * @throws GraphException
     */
    public IfNode(XmlElement nodeElement) throws GraphException {
        super(nodeElement);
    }

    /**
     * Returns the xpath.
     * 
     * @return The xpath
     */
    public String getXPath() {
        return this.xpath;
    }

    /**
     * Sets xpath.
     * 
     * @param xpath
     *            The xpath to set.
     */
    public void setXPath(String xpath) {
        this.xpath = xpath;
    }

    /**
     * @see edu.indiana.extreme.xbaya.graph.Node#getGUI()
     */
    public NodeGUI getGUI() {
        if (this.gui == null) {
            this.gui = new IfNodeGUI(this);
        }
        return this.gui;
    }

    /**
     * @see edu.indiana.extreme.xbaya.graph.impl.NodeImpl#getComponent()
     */
    @Override
    public IfComponent getComponent() {
        IfComponent component = (IfComponent) super.getComponent();
        if (component == null) {
            // The component is null when read from the graph XML.
            component = new IfComponent();
            setComponent(component);
        }
        return component;
    }

    /**
     * Adds additional input port.
     */
    public void addInputPort() {
        IfComponent component = getComponent();
        SystemComponentDataPort input = component.getInputPort();
        SystemDataPort port = input.createPort();
        addInputPort(port);
    }

    /**
     * @throws GraphException
     */
    public void removeInputPort() throws GraphException {
        List<DataPort> inputPorts = getInputPorts();
        // Remove the last one.
        DataPort inputPort = inputPorts.get(inputPorts.size() - 1);
        removeInputPort(inputPort);
    }

    @Override
    protected void parseConfiguration(XmlElement configElement) {
        super.parseConfiguration(configElement);
        XmlElement element = configElement.element(null, XPATH_TAG_NAME);
        if (element != null) {
            // TODO
            this.xpath = element.requiredText();
        }
    }

    @Override
    protected XmlElement toXML() {
        XmlElement nodeElement = super.toXML();
        nodeElement.setAttributeValue(GraphSchema.NS,
                GraphSchema.NODE_TYPE_ATTRIBUTE, GraphSchema.NODE_TYPE_IF);
        return nodeElement;
    }

    @Override
    protected XmlElement addConfigurationElement(XmlElement nodeElement) {
        XmlElement configElement = nodeElement.addElement(GraphSchema.NS,
                GraphSchema.NODE_CONFIG_TAG);
        if (this.xpath != null) {
            XmlElement element = configElement.addElement(GraphSchema.NS,
                    XPATH_TAG_NAME);
            element.addChild(this.xpath.toString());
        }
        return configElement;
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2006-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
